"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const os = __importStar(require("os"));
const uuid_1 = __importDefault(require("uuid"));
const GitCollector_1 = __importDefault(require("./GitCollector"));
const dynamic_require_1 = require("../utils/dynamic-require");
const config_1 = require("../config");
const IdCollector_1 = __importDefault(require("./IdCollector"));
const aspectoVersion = (_a = require('../../package.json')) === null || _a === void 0 ? void 0 : _a.version;
class Collector {
    constructor(ws, packageName, packageVersion) {
        this.ws = ws;
        this.collectableMap = {};
        this.intervalMap = {};
        let info = {};
        if (!packageName || !packageVersion) {
            if (!config_1.isNode) {
                throw new Error('Must provide packageName and packageVersion when running on browser');
            }
            try {
                info = JSON.parse(dynamic_require_1.fs()
                    .readFileSync('package.json')
                    .toString());
            }
            catch (e) {
                info = {
                    name: 'Failed to load package.json',
                    version: 'Failed to load package.json',
                };
            }
        }
        Collector.packageName = (packageName !== null && packageName !== void 0 ? packageName : info.name);
        Collector.packageVersion = (packageVersion !== null && packageVersion !== void 0 ? packageVersion : info.version);
    }
    static enrichData(data) {
        var _a, _b;
        data.env = process.env.NODE_ENV || 'empty';
        data.hostname = os.hostname();
        data.packageName = Collector.packageName;
        data.packageVersion = Collector.packageVersion;
        data.aspectoVersion = aspectoVersion;
        data.id = this.id;
        data.gitHash = (_b = (_a = GitCollector_1.default.hash, (_a !== null && _a !== void 0 ? _a : IdCollector_1.default.id)), (_b !== null && _b !== void 0 ? _b : undefined));
        return data;
    }
    addCollection(collect) {
        if (this.collectableMap[collect.getKey()]) {
            return false;
        }
        if (collect.interval() !== -1) {
            if (!this.intervalMap[collect.interval()]) {
                this.intervalMap[collect.interval()] = {
                    collectables: [collect],
                    sender: setInterval(() => {
                        this.sender(collect.interval());
                    }, collect.interval()),
                };
            }
            else {
                this.intervalMap[collect.interval()].collectables.push(collect);
            }
            this.sender(collect.interval());
        }
        else {
            this.sendData({
                [collect.getKey()]: collect.collect(),
            });
        }
        return true;
    }
    sender(interval) {
        const data = {};
        this.intervalMap[interval].collectables.forEach((collectable) => {
            data[collectable.getKey()] = collectable.collect();
        });
        this.sendData(data);
    }
    sendData(data) {
        data = Collector.enrichData(data);
        this.ws.send(data);
    }
}
exports.default = Collector;
Collector.id = uuid_1.default.v4();
//# sourceMappingURL=collector.js.map