"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../config");
const dynamic_require_1 = require("../utils/dynamic-require");
const tryGetTokenFromJson = (path) => {
    try {
        const aspectoConfig = JSON.parse(dynamic_require_1.fs()
            .readFileSync(path)
            .toString());
        return aspectoConfig.token;
    }
    catch (err) {
        return null;
    }
};
class AspectoConfig {
    static getToken(options) {
        var _a, _b;
        try {
            if ((_a = options) === null || _a === void 0 ? void 0 : _a.aspectoAuth) {
                return options.aspectoAuth;
            }
            if (process.env.ASPECTO_AUTH) {
                return process.env.ASPECTO_AUTH;
            }
            if (config_1.isNode) {
                if (tryGetTokenFromJson('aspecto.json')) {
                    return tryGetTokenFromJson('aspecto.json');
                }
                if (tryGetTokenFromJson(((_b = process) === null || _b === void 0 ? void 0 : _b.argv[1]) + '../aspecto.json')) {
                    return tryGetTokenFromJson(process.argv[1] + '../aspecto.json');
                }
            }
        }
        catch (e) {
            // Do nothing..
        }
        throw new Error('Unable to retrieve token, please provide it through ASPECTO_AUTH env param OR aspectoAuth init option OR in the root folder add aspecto.json with {"token":"-- token goes here --"}');
    }
    static isDevelopMode() {
        return process.env.ASPECTO_DEVELOP == 'true';
    }
    // Should be used in develop ONLY
    static getLinkedRootFolder() {
        return process.cwd() + '/';
    }
    // Should be used in develop ONLY
    static getLinkedNodeModule(moduleName) {
        return AspectoConfig.getLinkedRootFolder() + 'node_modules/' + moduleName;
    }
    static setSamplingRatio(ratio) {
        if (ratio) {
            AspectoConfig.samplingRatio = ratio;
        }
        else if (process.env.ASPECTO_SAMPLING_RATIO) {
            try {
                if (!isNaN(parseFloat(process.env.ASPECTO_SAMPLING_RATIO))) {
                    console.warn('process.env.ASPECTO_SAMPLING_RATIO must be a valid number');
                }
                else {
                    AspectoConfig.samplingRatio = parseFloat(process.env.ASPECTO_SAMPLING_RATIO);
                }
            }
            catch (e) {
                console.warn('process.env.ASPECTO_SAMPLING_RATIO must be a valid number');
            }
        }
    }
    static getSamplingRatio() {
        return AspectoConfig.samplingRatio;
    }
    static getNPMPostInstallId() {
        if (AspectoConfig.POST_INSTALL_ID) {
            return AspectoConfig.POST_INSTALL_ID;
        }
        else {
            try {
                const idJson = require('../id.json');
                AspectoConfig.POST_INSTALL_ID = idJson.id;
                return AspectoConfig.POST_INSTALL_ID;
            }
            catch (_a) {
                return undefined;
            }
        }
    }
}
exports.default = AspectoConfig;
AspectoConfig.samplingRatio = 1;
AspectoConfig.POST_INSTALL_ID = undefined;
//# sourceMappingURL=aspectoConfig.js.map