"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ws_1 = __importDefault(require("./communication/ws"));
const collector_1 = __importDefault(require("./collection/collector"));
const OsCollector_1 = __importDefault(require("./collection/OsCollector"));
const NpmCollector_1 = __importDefault(require("./collection/NpmCollector"));
const GitCollector_1 = __importDefault(require("./collection/GitCollector"));
const ExpressReporter_1 = __importDefault(require("./reporters/ExpressReporter"));
const NodeHttpReporter_1 = __importDefault(require("./reporters/NodeHttpReporter"));
const aspectoConfig_1 = __importDefault(require("./config/aspectoConfig"));
const ErrorReporter_1 = __importDefault(require("./reporters/ErrorReporter"));
const XMLHttpReporter_1 = __importDefault(require("./reporters/XMLHttpReporter"));
const IdCollector_1 = __importDefault(require("./collection/IdCollector"));
const config_1 = require("./config");
const ConfigService_1 = __importDefault(require("./config/ConfigService"));
exports.init = (options) => {
    const initPromise = (() => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f, _g;
        try {
            aspectoConfig_1.default.blacklist = (_a = options) === null || _a === void 0 ? void 0 : _a.blacklist;
            if ((_b = options) === null || _b === void 0 ? void 0 : _b.samplingRatio) {
                aspectoConfig_1.default.setSamplingRatio(options.samplingRatio);
            }
            else {
                aspectoConfig_1.default.setSamplingRatio();
            }
            const reporters = [];
            if (config_1.isNode) {
                reporters.push(new ExpressReporter_1.default());
                reporters.push(new ErrorReporter_1.default());
                if (((_c = options) === null || _c === void 0 ? void 0 : _c.outgoing) !== false) {
                    reporters.push(new NodeHttpReporter_1.default());
                }
            }
            else {
                if (((_d = options) === null || _d === void 0 ? void 0 : _d.outgoing) !== false) {
                    reporters.push(new XMLHttpReporter_1.default());
                }
            }
            const auth = aspectoConfig_1.default.getToken(options);
            yield ConfigService_1.default.init({ token: auth });
            const config = yield ConfigService_1.default.config;
            let ws = new ws_1.default(config_1.getCollectorEndpoint(((_e = options) === null || _e === void 0 ? void 0 : _e.sendTrafficTo) || config.collectorUrl));
            reporters.forEach((c) => c.set(ws, auth));
            const allowed = yield ws.auth(auth);
            if (!allowed) {
                console.error("Aspecto wasn't able to authenticate on your behalf");
                ws.close();
                ConfigService_1.default.close();
                return { close: () => { } };
            }
            const collector = new collector_1.default(ws, (_f = options) === null || _f === void 0 ? void 0 : _f.packageName, (_g = options) === null || _g === void 0 ? void 0 : _g.packageVersion);
            collector.addCollection(new OsCollector_1.default());
            if (config_1.isNode) {
                collector.addCollection(new NpmCollector_1.default());
                collector.addCollection(new GitCollector_1.default());
            }
            else {
                collector.addCollection(new IdCollector_1.default());
            }
            return {
                close: () => {
                    ws.close();
                    ConfigService_1.default.close();
                },
            };
        }
        catch (e) {
            console.error(`Failed to init Aspecto - ${e.message}`);
            return {
                close: () => { },
            };
        }
    }))();
    return {
        close() {
            return __awaiter(this, void 0, void 0, function* () {
                (yield initPromise).close();
            });
        },
    };
};
//# sourceMappingURL=index.js.map