"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const collector_1 = __importDefault(require("../collection/collector"));
const privacy_rules_1 = require("@aspecto/privacy-rules");
const ActionResult_1 = require("@aspecto/privacy-rules/dist/ActionResult");
const ConfigService_1 = __importDefault(require("../config/ConfigService"));
const BlacklistMatcher_1 = __importDefault(require("../config/BlacklistMatcher"));
class BaseReporter {
    constructor() {
        this.initPrivacyEngine();
    }
    initPrivacyEngine() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.privacyEngine) {
                const { privacyRules } = yield ConfigService_1.default.config;
                const privacyEngine = new privacy_rules_1.PrivacyEngine();
                privacyEngine.setRules(privacyRules);
                this.privacyEngine = privacyEngine;
            }
        });
    }
    sendData(report) {
        var _a, _b;
        // do not send anything until we initialize privacy rules
        if (!this.privacyEngine)
            return;
        let data = {};
        data = collector_1.default.enrichData(data);
        data.token = BaseReporter.token;
        data[this.getKey()] = report;
        if (BlacklistMatcher_1.default.match(data))
            return;
        const result = this.privacyEngine.executeRules(data);
        if (result === ActionResult_1.ActionResult.BlockRequest)
            return;
        // console.log(JSON.stringify(data));
        (_b = (_a = BaseReporter) === null || _a === void 0 ? void 0 : _a.ws) === null || _b === void 0 ? void 0 : _b.send(data);
    }
    set(ws, token) {
        BaseReporter.token = token;
        BaseReporter.ws = ws;
    }
}
exports.default = BaseReporter;
//# sourceMappingURL=BaseReporter.js.map