"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const BaseReporter_1 = __importDefault(require("./BaseReporter"));
class ErrorReporter extends BaseReporter_1.default {
    constructor() {
        super();
        this.multipleResolvesHandler = this.multipleResolvesHandler.bind(this);
        this.unhandledRejectionHandler = this.unhandledRejectionHandler.bind(this);
        this.uncaughtExceptionHandler = this.uncaughtExceptionHandler.bind(this);
        this.warningHandler = this.warningHandler.bind(this);
        process.on('multipleResolves', this.multipleResolvesHandler);
        process.on('unhandledRejection', this.unhandledRejectionHandler);
        process.on('uncaughtException', this.uncaughtExceptionHandler);
        process.on('warning', this.warningHandler);
    }
    getKey() {
        return 'NODE_ERROR';
    }
    warningHandler(warning) {
        this.sendData({
            type: 'warning',
            warning,
        });
    }
    unhandledRejectionHandler(reason, p) {
        this.sendData({
            type: 'unhandledRejection',
            promise: p,
            promiseReason: reason,
        });
    }
    uncaughtExceptionHandler(error) {
        this.sendData({
            type: 'uncaughtException',
            errorMessage: error.message,
            errorStack: error.stack,
        });
        throw error;
    }
    multipleResolvesHandler(type, promise, reason) {
        this.sendData({
            type: 'multipleResolves',
            promiseType: type,
            promise,
            promiseReason: reason,
        });
    }
}
exports.default = ErrorReporter;
//# sourceMappingURL=ErrorReporter.js.map