"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = __importDefault(require("uuid"));
const BaseReporter_1 = __importDefault(require("./BaseReporter"));
const aspectoConfig_1 = __importDefault(require("../config/aspectoConfig"));
const dynamic_require_1 = require("../utils/dynamic-require");
const path = __importStar(require("path"));
//@ts-ignore
const require_in_the_middle_1 = __importDefault(require("require-in-the-middle"));
var http;
const patchExpress = function (express) {
    var _a;
    if (((_a = express) === null || _a === void 0 ? void 0 : _a.isAspectoPatched) === true) {
        return;
    }
    var origin = express.application.init;
    express.application.init = function () {
        let value = origin.call(this);
        this.use((req, res, next) => {
            if (Math.random() >= aspectoConfig_1.default.getSamplingRatio()) {
                next();
                return;
            }
            res.locals.aspecto = {
                uuid: uuid_1.default.v4(),
                startTiming: new Date(),
            };
            var oldWrite = res.write, oldEnd = res.end;
            let chunks = [];
            res.write = function (chunk) {
                if (chunks.length <= 524288)
                    chunks.push(chunk);
                oldWrite.apply(res, arguments);
            };
            res.end = function (chunk) {
                if (chunk) {
                    chunks.push(chunk);
                }
                let body;
                try {
                    body = Buffer.concat(chunks).toString('utf8');
                }
                catch (_a) {
                    body = chunks[0];
                }
                let responseBody = {};
                if (body) {
                    try {
                        responseBody = JSON.parse(body);
                    }
                    catch (_b) {
                        responseBody = body;
                    }
                }
                oldEnd.apply(res, arguments);
                const endTime = new Date();
                http.sendData({
                    type: 'incoming',
                    httpClient: 'Express',
                    method: req.method,
                    httpVersion: req.httpVersion,
                    url: req.path,
                    body: responseBody,
                    requestBody: req.body,
                    fullUrl: req.url,
                    query: req.query,
                    params: req.params,
                    requestHeaders: req.headers,
                    responseHeaders: res.getHeaders(),
                    hostname: req.hostname,
                    ip: req.ip,
                    uuid: res.locals.aspecto.uuid,
                    protocol: req.protocol,
                    statusCode: res.statusCode,
                    executionTime: endTime.getTime() - res.locals.aspecto.startTiming.getTime(),
                    route: req.route,
                });
            };
            next();
        });
        let aspectoErrorMiddlewareHandler = function () {
            this.use(function (err, _req, res, next) {
                http.sendData({
                    isErrorMiddleware: true,
                    statusCode: res.statusCode,
                    body: res.body,
                    responseHeaders: res.headers,
                    uuid: res.locals.aspecto.uuid,
                    errorMessage: err.message,
                    errorStack: err.stack,
                });
                next(err);
            });
        };
        aspectoErrorMiddlewareHandler = aspectoErrorMiddlewareHandler.bind(this);
        setTimeout(aspectoErrorMiddlewareHandler, 2000);
        return value;
    };
    express.isAspectoPatched = true;
};
class ExpressReporter extends BaseReporter_1.default {
    constructor() {
        super();
        http = this;
        try {
            require_in_the_middle_1.default(['express'], function (express, _name, basedir) {
                const version = require(path.join(basedir, 'package.json')).version;
                express._version = version;
                patchExpress(express);
                return express;
            });
            patchExpress(dynamic_require_1.express());
        }
        catch (e) { }
    }
    getKey() {
        return 'HTTP_REQUEST';
    }
}
exports.default = ExpressReporter;
//# sourceMappingURL=ExpressReporter.js.map