"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = __importDefault(require("uuid"));
const BaseReporter_1 = __importDefault(require("./BaseReporter"));
const aspectoConfig_1 = __importDefault(require("../config/aspectoConfig"));
const dynamic_require_1 = require("../utils/dynamic-require");
const query_parser_1 = require("../utils/query-parser");
const url_patterns_1 = require("../utils/url-patterns");
const report_cache_1 = __importDefault(require("../utils/report-cache"));
let aspectoHttpClient;
const cloneResponse = (res) => {
    const PassThrough = dynamic_require_1.stream().PassThrough;
    const clone = new PassThrough();
    dynamic_require_1.mimic_response()(res, clone);
    return res.pipe(clone);
};
const secondsInterval = 300; // 5 minutes
const reportCache = new report_cache_1.default(secondsInterval);
const newRequest = (oldRequest, protocol) => (options, callback = (_res) => { }) => {
    const startTime = Date.now();
    const requestBodyChunks = [];
    const newCallback = (res) => {
        let body = [];
        const resClone = cloneResponse(res);
        resClone.setEncoding('utf8');
        resClone
            .on('data', (chunk) => {
            body.push(chunk);
        })
            .on('end', () => {
            var _a, _b, _c, _d;
            const host = (_d = (_c = (_b = (_a = res) === null || _a === void 0 ? void 0 : _a.connection) === null || _b === void 0 ? void 0 : _b._host, (_c !== null && _c !== void 0 ? _c : options.hostname)), (_d !== null && _d !== void 0 ? _d : options.host));
            const reportCount = reportCache.getReportCount(host);
            if (reportCount <= 0)
                return;
            body = body.join();
            try {
                const bodyAsJson = JSON.parse(body);
                body = bodyAsJson;
            }
            catch (_e) { }
            let requestBody = requestBodyChunks.join() ? requestBodyChunks.join() : {};
            try {
                requestBody = JSON.parse(requestBody);
            }
            catch (_f) { }
            const port = [443, 80].includes(res.connection.remotePort) ? null : res.connection.remotePort;
            const fullUrl = `${protocol}://${host}${port ? `:${port}` : ``}${options.path}`;
            const URL = dynamic_require_1.url().URL;
            const url = new URL(fullUrl);
            const patterns = url_patterns_1.findPatterns(url.pathname);
            const aspectoPayload = {
                type: 'outgoing',
                httpClient: protocol,
                method: options.method.toUpperCase(),
                httpVersion: res.httpVersion,
                fullUrl,
                url: url.pathname,
                body,
                requestBody,
                query: url.search ? query_parser_1.queryToObject(url.search) : {},
                params: patterns.params,
                requestHeaders: options.headers,
                responseHeaders: res.headers,
                port: options.port,
                ip: res.connection.remoteAddress,
                uuid: uuid_1.default(),
                hostname: url.host,
                protocol: `${protocol}:`,
                statusCode: res.statusCode,
                executionTime: Date.now() - startTime,
                route: { path: patterns.path },
                intervalAggregation: {
                    requestsInInterval: reportCount,
                    secondsInterval,
                },
            };
            aspectoHttpClient.sendData(aspectoPayload);
        });
        return callback.call(this, res);
    };
    // TODO: Support string options
    if (Math.random() >= aspectoConfig_1.default.getSamplingRatio() || typeof options === 'string') {
        return oldRequest(options, callback);
    }
    const clientRequest = oldRequest(options, newCallback);
    let oldWrite = clientRequest.write;
    clientRequest.write = (data) => {
        requestBodyChunks.push(data.toString());
        return oldWrite.call(clientRequest, data);
    };
    return clientRequest;
};
class OutgoingHttpReporter extends BaseReporter_1.default {
    constructor() {
        super();
        aspectoHttpClient = this;
        try {
            const httpInstance = dynamic_require_1.http();
            const oldHttpRequest = httpInstance.request;
            httpInstance.request = newRequest(oldHttpRequest, 'http');
            const httpsInstance = dynamic_require_1.https();
            const oldHttpsRequest = httpsInstance.request;
            httpsInstance.request = newRequest(oldHttpsRequest, 'https');
        }
        catch (err) {
            // Do nothing..
        }
    }
    getKey() {
        return 'HTTP_REQUEST';
    }
}
exports.default = OutgoingHttpReporter;
//# sourceMappingURL=NodeHttpReporter.js.map