"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const BaseReporter_1 = __importDefault(require("./BaseReporter"));
const query_parser_1 = require("../utils/query-parser");
const url_patterns_1 = require("../utils/url-patterns");
const aspectoConfig_1 = __importDefault(require("../config/aspectoConfig"));
const uuid_1 = __importDefault(require("uuid"));
let aspectoHttpClient;
const report = (method, urlString, requestBody, statusCode, responseBody, responseHeaders, requestHeaders, startTime) => {
    var _a;
    const url = new URL(urlString);
    const patterns = url_patterns_1.findPatterns(url.pathname);
    const query = url.search ? query_parser_1.queryToObject(url.search) : {};
    let requestBodyAsJson = (requestBody !== null && requestBody !== void 0 ? requestBody : {});
    try {
        requestBodyAsJson = JSON.parse(requestBody);
    }
    catch (_b) { }
    let responseBodyAsJson = (responseBody !== null && responseBody !== void 0 ? responseBody : {});
    try {
        responseBodyAsJson = JSON.parse(responseBody);
    }
    catch (_c) { }
    const responseHeadersAsJson = {};
    responseHeaders
        .split('\n')
        .filter((header) => header)
        .forEach((header) => {
        var _a;
        const h = header.split(':');
        responseHeadersAsJson[h[0]] = (_a = h[1]) === null || _a === void 0 ? void 0 : _a.trim();
    });
    const aspectoPayload = {
        type: 'outgoing',
        httpClient: 'XMLHttpRequest',
        method: method.toUpperCase(),
        fullUrl: urlString,
        url: url.pathname,
        body: responseBodyAsJson,
        requestBody: requestBodyAsJson,
        query,
        params: patterns.params,
        requestHeaders,
        responseHeaders: responseHeadersAsJson,
        port: url.port,
        httpVersion: 'N/A',
        ip: 'N/A',
        uuid: uuid_1.default(),
        hostname: url.host,
        protocol: url.protocol,
        statusCode,
        executionTime: Date.now() - startTime,
        route: { path: patterns.path },
    };
    (_a = aspectoHttpClient) === null || _a === void 0 ? void 0 : _a.sendData(aspectoPayload);
};
class XMLHttpReporter extends BaseReporter_1.default {
    constructor() {
        super();
        aspectoHttpClient = this;
        try {
            ((xhr) => {
                if (Math.random() >= aspectoConfig_1.default.getSamplingRatio()) {
                    return;
                }
                const open = xhr.open;
                xhr.open = function (method, url, ..._args) {
                    const startTime = Date.now();
                    const requestHeaders = {};
                    const send = this.send;
                    this.send = function (data) {
                        const onreadystatechange = this
                            .onreadystatechange;
                        this.onreadystatechange = function () {
                            var _a, _b;
                            if (this.readyState == XMLHttpRequest.DONE) {
                                try {
                                    if (this.withCredentials) {
                                        requestHeaders.cookie = (_b = (_a = window) === null || _a === void 0 ? void 0 : _a.document) === null || _b === void 0 ? void 0 : _b.cookie;
                                    }
                                    report(method, url, data, this.status, this.response, this.getAllResponseHeaders(), requestHeaders, startTime);
                                }
                                catch (err) {
                                    // Do nothing..
                                }
                            }
                            return onreadystatechange && onreadystatechange.apply(this, arguments);
                        };
                        return send.apply(this, arguments);
                    };
                    const setRequestHeader = xhr.setRequestHeader;
                    this.setRequestHeader = function (header, value) {
                        requestHeaders[header] = value;
                        return setRequestHeader.apply(this, arguments);
                    };
                    return open.apply(this, arguments);
                };
            })(XMLHttpRequest.prototype);
        }
        catch (err) {
            // Do nothing..
        }
    }
    getKey() {
        return 'HTTP_REQUEST';
    }
}
exports.default = XMLHttpReporter;
//# sourceMappingURL=XMLHttpReporter.js.map